@@ -85,7 +85,7 @@
 #define   MCHelpSwILOG       L"\n  ilog[name]    Log errors to file"
 #define   MCHelpSwINUL       L"\n  inul          Disable all messages"
 #define   MCHelpSwIOFF       L"\n  ioff[n]       Turn PC off after completing an operation"
-#define   MCHelpSwISND       L"\n  isnd          Enable sound"
+#define   MCHelpSwISND       L"\n  isnd          Control notification sounds"
 #define   MCHelpSwIVER       L"\n  iver          Display the version number"
 #define   MCHelpSwK          L"\n  k             Lock archive"
 #define   MCHelpSwKB         L"\n  kb            Keep broken extracted files"
@@ -127,10 +127,10 @@
 #define   MCHelpSwT          L"\n  t             Test files after archiving"
 #define   MCHelpSwTK         L"\n  tk            Keep original archive time"
 #define   MCHelpSwTL         L"\n  tl            Set archive time to latest file"
-#define   MCHelpSwTN         L"\n  tn[m,c,a]<t>  Process files newer than <t> time"
-#define   MCHelpSwTO         L"\n  to[m,c,a]<t>  Process files older than <t> time"
-#define   MCHelpSwTA         L"\n  ta[m,c,a]<d>  Process files modified after <d> YYYYMMDDHHMMSS date"
-#define   MCHelpSwTB         L"\n  tb[m,c,a]<d>  Process files modified before <d> YYYYMMDDHHMMSS date"
+#define   MCHelpSwTN         L"\n  tn[mcao]<t>   Process files newer than <t> time"
+#define   MCHelpSwTO         L"\n  to[mcao]<t>   Process files older than <t> time"
+#define   MCHelpSwTA         L"\n  ta[mcao]<d>   Process files modified after <d> YYYYMMDDHHMMSS date"
+#define   MCHelpSwTB         L"\n  tb[mcao]<d>   Process files modified before <d> YYYYMMDDHHMMSS date"
 #define   MCHelpSwTS         L"\n  ts[m,c,a]     Save or restore file time (modification, creation, access)"
 #define   MCHelpSwU          L"\n  u             Update files"
 #define   MCHelpSwV          L"\n  v             Create volumes with size autodetection or list all volumes"
